@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import argparse
 import json
 import textwrap
-from typing import Callable, Union
+from typing import Any, Callable
 
 from stairlight.stairlight import ResponseType, StairLight
 
@@ -19,10 +21,7 @@ def command_init(stairlight: StairLight, args: argparse.Namespace) -> str:
     message = ""
     stairlight_template_file = stairlight.init()
     if stairlight_template_file:
-        message = (
-            f"'{stairlight_template_file}' has created.\n"
-            "Please edit it to set your data sources."
-        )
+        message = stairlight_template_file
     return message
 
 
@@ -39,18 +38,14 @@ def command_check(stairlight: StairLight, args: argparse.Namespace) -> str:
     message = ""
     mapping_template_file = stairlight.check()
     if mapping_template_file:
-        message = (
-            f"'{mapping_template_file}' has created.\n"
-            "Please map undefined tables and parameters, "
-            "and append to your latest configuration file."
-        )
-    elif not stairlight.unmapped:
-        message = "Templates are not found."
+        message = mapping_template_file
 
     return message
 
 
-def command_up(stairlight: StairLight, args: argparse.Namespace) -> Union[dict, list]:
+def command_up(
+    stairlight: StairLight, args: argparse.Namespace
+) -> dict[str, Any] | list[dict[str, Any]]:
     """Execute up command
 
     Args:
@@ -58,13 +53,18 @@ def command_up(stairlight: StairLight, args: argparse.Namespace) -> Union[dict,
         args (argparse.Namespace): CLI arguments
 
     Returns:
-        Union[dict, list]: Upstairs results
+        dict[str, Any] | list[dict[str, Any]]: Upstairs results
     """
-    tables = get_tables_to_search(stairlight, args)
-    return execute_up_or_down(stairlight.up, args, tables)
+    return search(
+        func=stairlight.up,
+        args=args,
+        tables=find_tables_to_search(stairlight=stairlight, args=args),
+    )
 
 
-def command_down(stairlight: StairLight, args: argparse.Namespace) -> Union[dict, list]:
+def command_down(
+    stairlight: StairLight, args: argparse.Namespace
+) -> dict[str, Any] | list[dict[str, Any]]:
     """Execute down command
 
     Args:
@@ -72,28 +72,31 @@ def command_down(stairlight: StairLight, args: argparse.Namespace) -> Union[dict
         args (argparse.Namespace): CLI arguments
 
     Returns:
-        Union[dict, list]: Downstairs results
+        dict[str, Any] | list[dict[str, Any]]: Downstairs results
     """
-    tables = get_tables_to_search(stairlight, args)
-    return execute_up_or_down(stairlight.down, args, tables)
+    return search(
+        func=stairlight.down,
+        args=args,
+        tables=find_tables_to_search(stairlight=stairlight, args=args),
+    )
 
 
-def execute_up_or_down(
-    up_or_down: Callable, args: argparse.Namespace, tables: list
-) -> Union[dict, list]:
-    """Execute a command, up or down
+def search(
+    func: Callable, args: argparse.Namespace, tables: str | list[str]
+) -> dict[str, Any] | list[dict[str, Any]]:
+    """Search tables by executing stairlight.up() or stairlight.down()
 
     Args:
-        up_or_down (Callable): Either command_up() or command_down()
+        func (Callable): Either stairlight.up() or stairlight.down()
         args (argparse.Namespace): CLI arguments
-        tables (Union[str, list]): Tables to search
+        tables (str | list[str]): Tables to search
 
     Returns:
-        Union[dict, list]: Results
+        dict[str, Any] | list[dict[str, Any]]: Results
     """
     results = []
     for table_name in tables:
-        result = up_or_down(
+        result = func(
             table_name=table_name,
             recursive=args.recursive,
             verbose=args.verbose,
@@ -106,21 +109,23 @@ def execute_up_or_down(
     return results
 
 
-def get_tables_to_search(stairlight: StairLight, args: argparse.Namespace) -> list:
-    """Get tables to search
+def find_tables_to_search(
+    stairlight: StairLight, args: argparse.Namespace
+) -> list[str]:
+    """Find tables to search
 
     Args:
         stairlight (StairLight): Stairlight class
         args (argparse.Namespace): CLI arguments
 
     Returns:
-        list: Tables to search
+        list[str]: Tables to search
     """
     tables_to_search = []
     if args.table:
         tables_to_search = args.table
     elif args.label:
-        tables_to_search = stairlight.get_tables_by_labels(args.label)
+        tables_to_search = stairlight.find_tables_by_labels(target_labels=args.label)
         if not tables_to_search:
             exit()
     return tables_to_search
@@ -156,7 +161,12 @@ def set_save_load_parser(parser: argparse.ArgumentParser) -> None:
     """
     parser.add_argument(
         "--save",
-        help="file path where results will be saved(File system or GCS)",
+        help=textwrap.dedent(
+            """\
+            A file path where map results will be saved.
+            You can choose from local file system, GCS, S3.
+        """
+        ),
         type=str,
         default=None,
     )
@@ -164,15 +174,16 @@ def set_save_load_parser(parser: argparse.ArgumentParser) -> None:
         "--load",
         help=textwrap.dedent(
             """\
-            file path in which results are saved(File system or GCS),
-            can be specified multiple times
+            A file path where map results are saved.
+            You can choose from local file system, GCS, S3.
+            It can be specified multiple times.
         """
         ),
         action="append",
     )
 
 
-def set_up_down_parser(parser: argparse.ArgumentParser) -> None:
+def set_search_parser(parser: argparse.ArgumentParser) -> None:
     """Set arguments used by up and down
 
     Args:
@@ -234,8 +245,8 @@ def create_parser() -> argparse.ArgumentParser:
         argparse.ArgumentParser: ArgumentParser
     """
     description = (
-        "A table-level data lineage tool, "
-        "detects table dependencies by SELECT queries. "
+        "An end-to-end data lineage tool, "
+        "detects table dependencies by SQL SELECT statements. "
         "Without positional arguments, "
         "return a table dependency map as JSON format."
     )
@@ -252,30 +263,32 @@ def create_parser() -> argparse.ArgumentParser:
     parser_init.set_defaults(handler=command_init)
     set_general_parser(parser=parser_init)
 
-    # check
+    # map | check
     parser_check = subparsers.add_parser(
-        "check", help="create new configuration file about undefined mappings"
+        "map",
+        aliases=["check"],
+        help="create new configuration file about undefined mappings",
     )
     parser_check.set_defaults(handler=command_check)
     set_general_parser(parser=parser_check)
 
     # up
     parser_up = subparsers.add_parser(
-        "up", help="return upstairs ( table | SQL file ) list"
+        "up", help="return upstairs ( table | file ) list"
     )
     parser_up.set_defaults(handler=command_up)
     set_general_parser(parser=parser_up)
     set_save_load_parser(parser=parser_up)
-    set_up_down_parser(parser=parser_up)
+    set_search_parser(parser=parser_up)
 
     # down
     parser_down = subparsers.add_parser(
-        "down", help="return downstairs ( table | SQL file ) list"
+        "down", help="return downstairs ( table | file ) list"
     )
     parser_down.set_defaults(handler=command_down)
     set_general_parser(parser=parser_down)
     set_save_load_parser(parser=parser_down)
-    set_up_down_parser(parser=parser_down)
+    set_search_parser(parser=parser_down)
 
     return parser
 
@@ -301,7 +314,7 @@ def main() -> None:
             exit(f"'{args.config}/stairlight.y(a)ml' is not found.")
         result = stairlight.mapped
 
-    if args.quiet:
+    if args.quiet or not result:
         return
 
     if result and isinstance(result, str):
