@@ -1,19 +1,29 @@
+from __future__ import annotations
+
 import enum
 import json
-import os
 from logging import getLogger
-from typing import Union
+from typing import Any
 
-from . import config_key, map_key
-from .config import Configurator
+from .configurator import (
+    MAPPING_CONFIG_PREFIX_DEFAULT,
+    STAIRLIGHT_CONFIG_PREFIX_DEFAULT,
+    Configurator,
+)
 from .map import Map
-from .source.gcs import GCS_URI_SCHEME, get_gcs_blob
+from .source.config import (
+    MapKey,
+    MappingConfig,
+    StairlightConfig,
+    StairlightConfigSettings,
+)
+from .source.controller import LoadMapController, SaveMapController
 
 logger = getLogger(__name__)
 
 
 class ResponseType(enum.Enum):
-    """Enum: Execution result type of up/down command"""
+    """Enum: Execution result type of up|down command"""
 
     TABLE: str = "table"
     FILE: str = "file"
@@ -25,8 +35,8 @@ class ResponseType(enum.Enum):
 class SearchDirection(enum.Enum):
     """Enum: Search direction"""
 
-    UP = "Upstairs"
-    DOWN = "Downstairs"
+    UP: str = "Upstairs"
+    DOWN: str = "Downstairs"
 
     def __str__(self):
         return self.name
@@ -35,18 +45,21 @@ class SearchDirection(enum.Enum):
 class Node:
     """A Node of singly-linked list"""
 
-    def __init__(self, val, next=None):
-        self.val = val
-        self.next = None
+    def __init__(self, val: str):
+        self.val: str = val
+        self.next: "Node" | None = None
 
 
 class StairLight:
-    """Table dependency detector"""
+    """A table dependency detector"""
 
     def __init__(
-        self, config_dir: str = ".", load_files: list = None, save_file: str = None
+        self,
+        config_dir: str = ".",
+        load_files: list[str] = None,
+        save_file: str = "",
     ) -> None:
-        """Table dependency detector
+        """A table dependency detector
 
         Args:
             config_dir (str, optional):
@@ -57,13 +70,13 @@ class StairLight:
                 A file name of saving results if save option set. Defaults to None.
         """
         self.load_files = load_files
-        self.save_file = save_file
+        self.save_file: str = save_file
         self._configurator = Configurator(dir=config_dir)
-        self._mapped = {}
-        self._unmapped: list[dict] = []
-        self.mapping_config = None
-        self._stairlight_config = self._configurator.read(
-            prefix=config_key.STAIRLIGHT_CONFIG_FILE_PREFIX
+        self._mapped: dict[str, Any] = {}
+        self._unmapped: list[dict[str, Any]] = []
+        self._mapping_config: MappingConfig | None = None
+        self._stairlight_config: StairlightConfig = self._configurator.read_stairlight(
+            prefix=STAIRLIGHT_CONFIG_PREFIX_DEFAULT
         )
 
     def create_map(self) -> None:
@@ -80,7 +93,7 @@ class StairLight:
             self.save_map()
 
     @property
-    def mapped(self) -> dict:
+    def mapped(self) -> dict[str, Any]:
         """Return mapped
 
         Returns:
@@ -89,7 +102,7 @@ class StairLight:
         return self._mapped
 
     @property
-    def unmapped(self) -> "list[dict]":
+    def unmapped(self) -> list[dict[str, Any]]:
         """Return unmapped
 
         Returns:
@@ -98,32 +111,32 @@ class StairLight:
         return self._unmapped
 
     def has_stairlight_config(self) -> bool:
-        """Exists stairlight configuration file or not
+        """Exists a stairlight configuration file or not
 
         Returns:
             bool: Exists stairlight configuration file or not
         """
-        return self._stairlight_config is not None
+        return len(self._stairlight_config.Include) > 0
 
     def _set_config(self) -> None:
-        """Set config"""
+        """Set configurations"""
         if not self._stairlight_config:
-            logger.warning(
-                f"{config_key.STAIRLIGHT_CONFIG_FILE_PREFIX}.y(a)ml' is not found."
-            )
+            logger.warning(f"{STAIRLIGHT_CONFIG_PREFIX_DEFAULT}.y(a)ml' is not found.")
             return
 
-        mapping_config_prefix = config_key.MAPPING_CONFIG_FILE_PREFIX
-        if config_key.STAIRLIGHT_CONFIG_SETTING_SECTION in self._stairlight_config:
-            settings = self._stairlight_config[
-                config_key.STAIRLIGHT_CONFIG_SETTING_SECTION
-            ]
-            if config_key.MAPPING_PREFIX in settings:
-                mapping_config_prefix = settings[config_key.MAPPING_PREFIX]
-        self._mapping_config = self._configurator.read(prefix=mapping_config_prefix)
+        mapping_config_prefix: str = MAPPING_CONFIG_PREFIX_DEFAULT
+        if self._stairlight_config.Settings:
+            settings: StairlightConfigSettings = StairlightConfigSettings(
+                **self._stairlight_config.Settings
+            )
+            if settings.MappingPrefix:
+                mapping_config_prefix = settings.MappingPrefix
+        self._mapping_config = self._configurator.read_mapping(
+            prefix=mapping_config_prefix
+        )
 
     def _write_map(self) -> None:
-        """write a dependency map"""
+        """Write a dependency map"""
         dependency_map = Map(
             stairlight_config=self._stairlight_config,
             mapping_config=self._mapping_config,
@@ -135,7 +148,7 @@ class StairLight:
 
         self._unmapped = dependency_map.unmapped
 
-    def init(self, prefix: str = config_key.STAIRLIGHT_CONFIG_FILE_PREFIX) -> str:
+    def init(self, prefix: str = STAIRLIGHT_CONFIG_PREFIX_DEFAULT) -> str:
         """Create Stairlight template file
 
         Args:
@@ -145,9 +158,9 @@ class StairLight:
         Returns:
             str: Template file name
         """
-        return self._configurator.create_stairlight_template_file(prefix=prefix)
+        return self._configurator.create_stairlight_file(prefix=prefix)
 
-    def check(self, prefix: str = config_key.MAPPING_CONFIG_FILE_PREFIX) -> str:
+    def check(self, prefix: str = MAPPING_CONFIG_PREFIX_DEFAULT) -> str:
         """Check mapped results and create a mapping template file
 
         Args:
@@ -159,71 +172,41 @@ class StairLight:
         """
         if self.load_files:
             logger.warning("Load option is used, skip checking.")
-            return None
+            return ""
         elif not self._unmapped:
-            return None
+            return ""
 
-        return self._configurator.create_mapping_template_file(
+        return self._configurator.create_mapping_file(
             unmapped=self._unmapped, prefix=prefix
         )
 
     def save_map(self) -> None:
         """Save mapped results"""
-        if self.save_file.startswith(GCS_URI_SCHEME):
-            self.save_map_gcs()
-        else:
-            self.save_map_fs()
-
-    def save_map_gcs(self) -> None:
-        """Save mapped results to Google Cloud Storage"""
-        blob = self.get_gcs_blob(self.save_file)
-        blob.upload_from_string(
-            data=json.dumps(obj=self._mapped, indent=2),
-            content_type="application/json",
+        save_map_controller = SaveMapController(
+            save_file=self.save_file, mapped=self._mapped
         )
-
-    def save_map_fs(self) -> None:
-        """Save mapped results to file system"""
-        with open(self.save_file, "w") as f:
-            json.dump(self._mapped, f, indent=2)
+        save_map_controller.save()
 
     def load_map(self) -> None:
         """Load mapped results"""
+        if not self.load_files:
+            return
         for load_file in self.load_files:
-            loaded_map = {}
-            if load_file.startswith(GCS_URI_SCHEME):
-                loaded_map = self.load_map_gcs(load_file=load_file)
-            else:
-                loaded_map = self.load_map_fs(load_file=load_file)
+            load_map_controller = LoadMapController(load_file=load_file)
+            loaded_map = load_map_controller.load()
 
             if self._mapped:
                 self._mapped = deep_merge(org=self._mapped, add=loaded_map)
             else:
                 self._mapped = loaded_map
 
-    def load_map_gcs(self, load_file: str) -> dict:
-        """Load mapped results from Google Cloud Storage"""
-        blob = get_gcs_blob(load_file)
-        if not blob.exists():
-            logger.error(f"{load_file} is not found.")
-            exit()
-        return json.loads(blob.download_as_string())
-
-    def load_map_fs(self, load_file: str) -> dict:
-        """Load mapped results from file system"""
-        if not os.path.exists(load_file):
-            logger.error(f"{load_file} is not found.")
-            exit()
-        with open(load_file) as f:
-            return json.load(f)
-
     def up(
         self,
         table_name: str,
         recursive: bool = False,
         verbose: bool = False,
         response_type: str = ResponseType.TABLE.value,
-    ) -> Union[list, dict]:
+    ) -> list[str] | dict[str, Any]:
         """Search upstream nodes
 
         Args:
@@ -234,7 +217,7 @@ class StairLight:
                 Response type. Defaults to ResponseType.TABLE.value.
 
         Returns:
-            Union[list, dict]: [description]
+            list[str] | dict[str, Any]: Search results
         """
         return self.search(
             table_name=table_name,
@@ -250,7 +233,7 @@ class StairLight:
         recursive=False,
         verbose=False,
         response_type=ResponseType.TABLE.value,
-    ) -> Union[list, dict]:
+    ) -> list[str] | dict[str, Any]:
         """Search downstream nodes
 
         Args:
@@ -261,7 +244,7 @@ class StairLight:
                 Response type. Defaults to ResponseType.TABLE.value.
 
         Returns:
-            Union[list, dict]: [description]
+            list[str] | dict[str, Any]: Search results
         """
         return self.search(
             table_name=table_name,
@@ -278,7 +261,7 @@ class StairLight:
         verbose: bool,
         response_type: str,
         direction: SearchDirection,
-    ) -> Union[list, dict]:
+    ) -> list[str] | dict[str, Any]:
         """Search nodes
 
         Args:
@@ -289,7 +272,7 @@ class StairLight:
             direction (SearchDirection): Search direction
 
         Returns:
-            Union[list, dict]: [description]
+            list[str] | dict[str, Any]: Search results
         """
         if verbose:
             return self.search_verbose(
@@ -310,14 +293,14 @@ class StairLight:
                 head=True,
             )
 
-        return None
+        return []
 
     def search_verbose(
         self,
         table_name: str,
         recursive: bool,
         direction: SearchDirection,
-        searched_tables: list,
+        searched_tables: list[str],
         head: bool,
     ) -> dict:
         """Search nodes and return verbose results
@@ -326,14 +309,16 @@ class StairLight:
             table_name (str): Table name
             recursive (bool): Search recursively or not
             direction (SearchDirection): Search direction
-            searched_tables (list): a list of searched tables
+            searched_tables (list[str]): a list of searched tables
             head (bool): Current position is head or not
 
         Returns:
             dict: Search results
         """
-        relative_map = self.get_relative_map(table_name, direction)
-        response = {table_name: {}}
+        relative_map = self.create_relative_map(
+            table_name=table_name, direction=direction
+        )
+        response: dict[str, Any] = {table_name: {}}
         if not relative_map:
             return response
 
@@ -370,7 +355,6 @@ class StairLight:
                 }
 
         response[table_name][direction.value] = relative_map
-        logger.debug(json.dumps(response, indent=2))
         return response
 
     def search_plain(
@@ -379,9 +363,9 @@ class StairLight:
         recursive: bool,
         response_type: str,
         direction: SearchDirection,
-        searched_tables: list,
+        searched_tables: list[str],
         head: bool,
-    ) -> list:
+    ) -> list[str]:
         """Search nodes and return simple results
 
         Args:
@@ -389,17 +373,20 @@ class StairLight:
             recursive (bool): Search recursively or not
             response_type (str): Response type value
             direction (SearchDirection): Search direction
-            searched_tables (list): a list of searched tables
+            searched_tables (list[str]): a list of searched tables
             head (bool): Current position is head or not
 
         Returns:
-            list: Search results
+            list[str]: Search results
         """
-        relative_map = self.get_relative_map(table_name, direction)
-        response = []
+        relative_map = self.create_relative_map(
+            table_name=table_name, direction=direction
+        )
+        response: list[str] = []
         if not relative_map:
             return response
 
+        next_table_name: str
         for next_table_name in relative_map.keys():
             if recursive:
                 if head:
@@ -430,13 +417,15 @@ class StairLight:
             if response_type == ResponseType.TABLE.value:
                 response.append(next_table_name)
             elif response_type == ResponseType.FILE.value:
-                response.append(relative_map[next_table_name].get(map_key.URI))
+                response.append(relative_map[next_table_name].get(MapKey.URI))
             logger.debug(json.dumps(response, indent=2))
 
         return sorted(list(set(response)))
 
-    def get_relative_map(self, table_name: str, direction: SearchDirection) -> dict:
-        """Get a relative map for the specified direction
+    def create_relative_map(
+        self, table_name: str, direction: SearchDirection
+    ) -> dict[str, Any]:
+        """Create a relative map for the specified direction
 
         Args:
             table_name (str): Table name
@@ -445,92 +434,126 @@ class StairLight:
         Returns:
             dict: Relative map
         """
-        relative_map = {}
+        relative_map: dict[str, Any] = {}
         if direction == SearchDirection.UP:
-            relative_map = self._mapped.get(table_name)
+            relative_map = self._mapped.get(table_name, {})
         elif direction == SearchDirection.DOWN:
             for key in [k for k, v in self._mapped.items() if v.get(table_name)]:
                 relative_map[key] = self._mapped[key][table_name]
         return relative_map
 
-    def get_tables_by_labels(self, targets: list) -> list:
-        """Get tables to search by labels
+    def find_tables_by_labels(self, target_labels: list[str]) -> list[str]:
+        """Find tables by labels
 
         Args:
-            targets (list): Target labels
+            target_labels (list[str]): Target labels
 
         Returns:
-            list: Tables to search
+            list[str]: Tables to search
         """
-        tables_to_search = []
+        tables_to_search: list[str] = []
 
         # "mapping" section in mapping.yaml
-        for configurations in self._mapping_config.get(
-            config_key.MAPPING_CONFIG_MAPPING_SECTION
-        ):
-            for table_attributes in configurations.get(config_key.TABLES):
-                if self.is_target_found(
-                    targets=targets,
-                    labels=table_attributes.get(config_key.LABELS, {}),
+        if not self._mapping_config:
+            return tables_to_search
+
+        for mapping in self._mapping_config.get_mapping():
+            for mapping_table in mapping.get_table():
+                if (
+                    mapping_table.TableName not in tables_to_search
+                    and self.is_target_label_found(
+                        target_labels=target_labels,
+                        configured_labels=mapping_table.Labels,
+                    )
                 ):
-                    tables_to_search.append(table_attributes[config_key.TABLE_NAME])
+                    tables_to_search.append(mapping_table.TableName)
 
         # "metadata" section in mapping.yaml
-        for table_attributes in self._mapping_config.get(
-            config_key.MAPPING_CONFIG_METADATA_SECTION
-        ):
-            if self.is_target_found(
-                targets=targets,
-                labels=table_attributes.get(config_key.LABELS, {}),
+        for metadata in self._mapping_config.get_metadata():
+            if (
+                metadata.TableName not in tables_to_search
+                and self.is_target_label_found(
+                    target_labels=target_labels,
+                    configured_labels=metadata.Labels,
+                )
             ):
-                tables_to_search.append(table_attributes[config_key.TABLE_NAME])
+                tables_to_search.append(str(metadata.TableName))
 
         return tables_to_search
 
     @staticmethod
-    def is_target_found(targets: list, labels: dict) -> bool:
-        result = False
-        found = 0
-        for label_key, label_value in labels.items():
-            for target in targets:
-                target_key = target.split(":")[0]
-                target_value = target.split(":")[1]
-                if target_key == label_key and target_value == label_value:
-                    found += 1
-        if found == len(targets):
-            result = True
-        return result
+    def is_target_label_found(
+        target_labels: list[str], configured_labels: dict[str, Any]
+    ) -> bool:
+        """Return a target label found or not.
+
+        Args:
+            target_labels (list[str]): Target labels
+            configured_labels (dict[str, Any]): Labels in configurations
+
+        Returns:
+            bool: A target label found or not.
+        """
+        found_count: int = 0
+        configured_label_key: str
+        configured_label_value: str
+        if not configured_labels:
+            return False
+        for configured_label_key, configured_label_value in configured_labels.items():
+            for target_label in target_labels:
+                target_label_key = target_label.split(":")[0]
+                target_label_value = target_label.split(":")[1]
+                if (
+                    target_label_key == configured_label_key
+                    and target_label_value == configured_label_value
+                ):
+                    found_count += 1
+
+        return found_count == len(target_labels)
 
 
-def is_cyclic(tables: list) -> bool:
+def is_cyclic(tables: list[str]) -> bool:
     """Floyd's cycle-finding algorithm
 
     Args:
-        tables (list): Detected tables
+        tables (list[str]): Detected tables
 
     Returns:
-        bool: Table dependencies are cyclic or not
+        bool: The table dependency is cyclic or not
     """
-    nodes = {}
+    nodes: dict[str, Node] = {}
     for table in tables:
         if table not in nodes.keys():
             nodes[table] = Node(table)
     for i, table in enumerate(tables):
         if not nodes[table].next and i < len(tables) - 1:
             nodes[table].next = nodes[tables[i + 1]]
+
+    slow: Node | None
+    fast: Node | None
     slow = fast = nodes[tables[0]]
     while fast and fast.next:
-        slow = slow.next
+        if slow:
+            slow = slow.next
         fast = fast.next.next
         if slow == fast:
             return True
     return False
 
 
-def deep_merge(org: dict, add: dict) -> dict:
-    new: dict = org
+def deep_merge(org: dict[str, Any], add: dict[str, Any]) -> dict[str, Any]:
+    """Merge nested dicts
+
+    Args:
+        org (dict[str, Any]): Original dict
+        add (dict[str, Any]): Dict to add
+
+    Returns:
+        dict: Merged dict
+    """
+    new: dict[str, Any] = org
     for add_key, add_value in add.items():
-        org_value = org.get(add_key)
+        org_value: dict[str, Any] = org.get(add_key, {})
         if add_key not in org:
             new[add_key] = add_value
         elif isinstance(add_value, dict):
